﻿Imports System
Imports System.Collections
Imports System.IO
Imports System.Xml
Imports System.Xml.Serialization


Public Class Comic
    Implements IDisposable


    Private _pages As ComicPages

    Public Property ArchiveType As ArchiveType
    Public Property TempFolder As String


    Private _fileList As New List(Of String)
    Private actualIndex As Integer = -1
    Private disposedValue As Boolean = False        ' To detect redundant calls
    Public readMePath As String = String.Empty

    Public Sub New(ByVal fileName As String)
        Me.IsFavorite = False
        Path = fileName
        If IO.File.Exists(Path) Then
            Dim ext As String = IO.Path.GetExtension(Me.Path)
            ext = Replace(ext, ".", "")
            Select Case LCase(Trim(ext))
                Case "cbz", "zip", "gzip"
                    ArchiveType = eComic2010.ArchiveType.CBZ
                Case "rar", "cbr"
                    ArchiveType = eComic2010.ArchiveType.CBR
                Case "pdf"
                    ArchiveType = eComic2010.ArchiveType.PDF
                Case Else
                    ArchiveType = eComic2010.ArchiveType.Unknown
                    If FileChecker.CheckSignature(fileName, 7, "52-61-72-21-1A-07-00") Then ArchiveType = eComic2010.ArchiveType.CBR : Exit Select ' RAR checking 
                    If FileChecker.CheckSignature(fileName, 3, "1F-8B-08") Then ArchiveType = eComic2010.ArchiveType.CBZ : Exit Select ' gzip checking 
                    If FileChecker.CheckSignature(fileName, 4, "50-4B-03-04") Then ArchiveType = eComic2010.ArchiveType.CBZ : Exit Select ' PK-Zip Checking 
            End Select ' Case LCase(Trim(ext)) 

            Me.FileName = IO.Path.GetFileNameWithoutExtension(fileName)
            Dim str As String = My.Computer.FileSystem.SpecialDirectories.CurrentUserApplicationData
            TempFolder = str & "/archives/" & IO.Path.GetFileNameWithoutExtension(fileName)
            If Not IO.Directory.Exists(TempFolder) Then IO.Directory.CreateDirectory(TempFolder)
        ElseIf IO.Directory.Exists(fileName) Then
            ArchiveType = eComic2010.ArchiveType.OpenDirectory
            Me.TempFolder = String.Empty
        Else
            Me.TempFolder = String.Empty
            Throw New FileNotFoundException
        End If

    End Sub

    Public Property FileName As String
    Public Property Path As String

    Public ReadOnly Property ComicPages() As ComicPages
        Get
            If _pages Is Nothing Then _pages = New ComicPages
            Return _pages
        End Get
    End Property


    Public Property IsFavorite As Boolean


  
#Region " IDisposable Support "

    Protected Sub cleanupFolders(ByVal folder As String)
        If Not String.IsNullOrEmpty(folder) AndAlso IO.Directory.Exists(folder) Then
            For Each dn As String In IO.Directory.GetDirectories(folder)
                cleanupFolders(dn)
            Next
            For Each fn As String In IO.Directory.GetFiles(folder)
                Try
                    IO.File.Decrypt(fn)
                Catch ex As Exception
                End Try
            Next
            Try
                IO.Directory.Delete(folder)
            Catch ex As Exception
            End Try
        End If
    End Sub
    ' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                ' TODO: free other state (managed objects).
            End If
            cleanupFolders(Me.TempFolder)
        End If
        Me.disposedValue = True
    End Sub


    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

End Class

Public Enum ArchiveType
    CBZ
    CBR
    PDF
    OpenDirectory
    Unknown
End Enum